//
//genre.c
//
//    v0.01 - original
//
//returns the genre information for a given channel
//only for 456:12.88
//
//this code returns the genre information for the current program on a given channel
//only for 456:12.88
//
//not a tap
//link into your own tap
//provides 3 functions:
//
//int exTAP_GetCurrentGenre(int chType, int chNum, int *nibble1, int *nibble2);
//int exTAP_GetCurrentGenreStr(int chType, int chNum, char *str);
//int exTAP_GetGenre(TYPE_exTapEvent *tapEvtInfo, int *nibble1, int *nibble2);
//
#include <tap.h>

//----------------------------------------------------------------------------
int exTAP_GetCurrentGenre(int chType, int chNum, int *nibble1, int *nibble2);
int exTAP_GetCurrentGenreStr(int chType, int chNum, char *str);
int exTAP_GetGenre(TYPE_TapEvent *tapEvtInfo, int *nibble1, int *nibble2);

//----------------------------------------------------------------------------
dword fw_sysid = 0;
extern int _appl_version;

//----------------------------------------------------------------------------
//
//fw_addresses
//

//12.88
dword fw_max_events = 0x000036b0;
dword fw_event_table_size = 0x00000044;
dword fw_event_table = 0x80333284;

//12.25
//dword fw_max_events = 0x00001388;
//dword fw_event_table_size = 0x00000040;
//dword fw_event_table = 0x8032a7c8;

//----------------------------------------------------------------------------
//
//error codes
//

//#define exTAPdebug

enum {
    EXTAPERR_OTHER              = -5,
    EXTAPERR_BAD_FIRMWARE       = -1,
    EXTAPERR_NONE               =  0,
};

//----------------------------------------------------------------------------
//this is the internal data structure used to store events
//it _always_ starts at address FW_EVENT_TABLE
//what are the unknown values ???
//how is the event_name, extended_event_name allocated ???
//note that the extended_event_name is a list of (num_extended) zero terminated (name,description) pairs
//with a total length of extended_length  The final pair is terminated with another 0 (i.e. 2 0's indicate end of list)
//
typedef struct{
    byte  char00;                   //00    // top 2 bits - valid; lower 5 bits - running status
                                            //pre 1288: top 2 bits = valid, next 1 bit = ???, next 5 bits = running status,
                                            //for 1288: top 2 bits = valid, next 3 bits = running status, next 3 = ???
    byte  char01;                   //01
    byte  duration_hour;            //02
    byte  duration_min;             //03
    word  short04;                  //04    //??? normally 0000
    word  event_id;                 //06
    word  start_date;               //08    //mjd, utc time
    byte  start_hour;               //0a
    byte  start_min;                //0b
    word  end_date;                 //0c    //mjd, utc time
    byte  end_hour;                 //0e
    byte  end_min;                  //0f
    byte  event_name_length;        //10
    byte  parental_rating;          //11
    word  original_network_id;      //12
    word  transport_stream_id;      //14
    word  service_id;               //16
    word  short18;                  //18    //??? normally 0000
    word  sat_index;                //1a
    word  short1c;                  //1c    //??? normally 0000
    word  short1e;                  //1e    //??? normally 0000    or 6500
    word  short20;                  //20    //??? normally 0000
    word  previous;                 //22    //previous, a link to the previous epg entry, by date/time, ffff terminates
    word  short24;                  //24    //??? normally 0000
    word  next;                     //26    //next, a link to the previous epg entry, by date/time, ffff terminates
    byte  *event_name;              //28    //ptr to event name, event description is offset by event_name_length bytes
    byte  char2c;                   //2c    //01 if extended present else 00
    byte  num_extended;             //2d    //number of extended items
    word  extended_length;          //2e    //extended length
    byte  *extended_event_name;     //30
    word  short34;                  //34    //0001 if extended present else 0000
    word  short36;                  //36    //0001 if extended present else 0000
    word  date_added;               //38    //mjd, utc time - date added ?
    word  short3a;                  //3a    //??? normally 0000
    word  short3c;                  //3c    //??? normally 0000
    word  short3e;                  //3e    //??? normally 0000

    //for fw:1288, this is 0x44 byte long
    byte  content_nibble1;          //40
    byte  content_nibble2;          //41
    word  short42;                  //42    //user_nibble1,2? - not filled in by fw?
}type_eventtable;

//----------------------------------------------------------------------------
//content_nibble1, content_nibble2
// level 2 does not appear to be used
#define NO_CONTENT_TYPES      80
struct {
    byte nibble1;
    byte nibble2;
    byte description[0x40];
} content[] = {
    { 0x0,0x00,"Undefined" },

    { 0x1,0x00,"Movie/Drama" },
    { 0x1,0x01,"detective/thriller" },
    { 0x1,0x02,"adventure/western/war" },
    { 0x1,0x03,"science fiction/fantasy/horror" },
    { 0x1,0x04,"comedy" },
    { 0x1,0x05,"soap/melodrama/folkloric" },
    { 0x1,0x06,"romance" },
    { 0x1,0x07,"serious/classical/religious/historical movie/drama" },
    { 0x1,0x08,"adult movie/drama" },

    { 0x2,0x00,"News/Current Affairs" },
    { 0x2,0x01,"news/weather report" },
    { 0x2,0x02,"news magazine" },
    { 0x2,0x03,"documentary" },
    { 0x2,0x04,"discussion/interview/debate" },

    { 0x3,0x00,"Show/Game Show" },
    { 0x3,0x01,"game show/quiz/contest" },
    { 0x3,0x02,"variety show" },
    { 0x3,0x03,"talk show" },

    { 0x4,0x00,"Sports" },
    { 0x4,0x01,"special sports events" },
    { 0x4,0x02,"sports magazines" },
    { 0x4,0x03,"football/soccer" },
    { 0x4,0x04,"tennis/squash" },
    { 0x4,0x05,"team sports" },
    { 0x4,0x06,"athletics" },
    { 0x4,0x07,"motor sport" },
    { 0x4,0x08,"water sport" },
    { 0x4,0x09,"winter sports" },
    { 0x4,0x0a,"equestrian" },
    { 0x4,0x0b,"martial sports" },

    { 0x5,0x00,"Children's/Youth Programmes" },
    { 0x5,0x01,"pre-school children's programmes" },
    { 0x5,0x02,"entertainment programmes for 6 to 14" },
    { 0x5,0x03,"entertainment programmes for 10 to 16" },
    { 0x5,0x04,"informational/educational/school programmes" },
    { 0x5,0x05,"cartoons/puppets" },

    { 0x6,0x00,"Music/Ballet/Dance" },
    { 0x6,0x01,"rock/pop" },
    { 0x6,0x02,"serious music/classical music" },
    { 0x6,0x03,"folk/traditional music" },
    { 0x6,0x04,"jazz" },
    { 0x6,0x05,"musical/opera" },
    { 0x6,0x06,"ballet" },

    { 0x7,0x00,"Arts/Culture (without music)" },
    { 0x7,0x01,"performing arts" },
    { 0x7,0x02,"fine arts" },
    { 0x7,0x03,"religion" },
    { 0x7,0x04,"popular culture/traditional arts" },
    { 0x7,0x05,"literature" },
    { 0x7,0x06,"film/cinema" },
    { 0x7,0x07,"experimental film/video" },
    { 0x7,0x08,"broadcasting/press" },
    { 0x7,0x09,"new media" },
    { 0x7,0x0a,"arts/culture magazines" },
    { 0x7,0x0b,"fashion" },

    { 0x8,0x00,"Social/Political Issues/Economics" },
    { 0x8,0x01,"magazines/reports/documentary" },
    { 0x8,0x02,"economics/social advisory" },
    { 0x8,0x03,"remarkable people" },

    { 0x9,0x00,"Education/Science/Factual Topics" },
    { 0x9,0x01,"nature/animals/environment" },
    { 0x9,0x02,"technology/natural sciences" },
    { 0x9,0x03,"medicine/physiology/psychology" },
    { 0x9,0x04,"foreign countries/expeditions" },
    { 0x9,0x05,"social/spiritual sciences" },
    { 0x9,0x06,"further education" },
    { 0x9,0x07,"languages" },

    { 0xa,0x00,"Leisure Hobbies" },
    { 0xa,0x01,"tourism/travel" },
    { 0xa,0x02,"handicraft" },
    { 0xa,0x03,"motoring" },
    { 0xa,0x04,"fitness & health" },
    { 0xa,0x05,"cooking" },
    { 0xa,0x06,"advertisement/shopping" },
    { 0xa,0x07,"gardening" },

    { 0xb,0x00,"Original Language" },
    { 0xb,0x01,"black and white" },
    { 0xb,0x02,"unpublished" },
    { 0xb,0x03,"live broadcast" },
};

//----------------------------------------------------------------------------
char *get_content_type_str(int nibble1, int nibble2)
{
    int i;

    for(i=1; i<NO_CONTENT_TYPES; i++) {
        if ((content[i].nibble1 == nibble1) && (content[i].nibble2 == nibble2)) {
            return content[i].description;
        }
    }
    return content[0].description;
}

//----------------------------------------------------------------------------
//only for 12.88
int exTAP_GetCurrentGenre(int chType, int chNum, int *nibble1, int *nibble2)
{
    TYPE_TapChInfo channel_info;
    type_eventtable *fw_et;
    int event_index;
    int i,j,k;
    int running_status;

    *nibble1 = 0;
    *nibble2 = 0;
    if (!fw_sysid) fw_sysid = *(word*)0xa3fffffa;
    if (!fw_max_events) goto exit;
    if (!fw_event_table_size) goto exit;
    if (!fw_event_table) goto exit;
    if ((fw_sysid==456)&&(_appl_version==0x1288)) {
        TAP_Channel_GetInfo(chType, chNum, &channel_info);

        //goto the start of the chain
        event_index = 0;
        k = 0;
        while (k < fw_max_events) {
            //the sizeof(fw_event_table) is different from 456:1288
            //so, don't use a static index, e.g. fw_event_table[i].
            fw_et = (type_eventtable *)((dword)fw_event_table + (event_index * fw_event_table_size));

            if ((fw_et->char00 & 0xc0) == 0) {
                //find a valid entry
                event_index++;
            }
            else {
                //is this the start
                if (fw_et->previous == 0xffff) {
                    break;
                }
                event_index = fw_et->previous;
            }
            k++;
        }
        if (k == fw_max_events) {
            //epg empty
#ifdef exTAPdebug
            TAP_Print("\nepg empty");
#endif
            return EXTAPERR_OTHER;
        }

        j = event_index;
        k = 0;
        //find the curent program
        while(k < fw_max_events) {
            fw_et = (type_eventtable *)((dword)fw_event_table + (j * fw_event_table_size));

            //match service_id
            if (fw_et->service_id == channel_info.svcId) {
//                running_status = (fw_et->char00 >> 0) & 0x1f;   //pre12.88
                running_status = (fw_et->char00 >> 3) & 7;      //12.88
                if (running_status == 4) {
                    //get the genre type
                    *nibble1 = fw_et->content_nibble1;
                    *nibble2 = fw_et->content_nibble2;
                    break;
                }
            }
            j = fw_et->next;
            if (j == 0xffff) {
#ifdef exTAPdebug
                TAP_Print("\ncurrent program not found");
#endif
                return EXTAPERR_OTHER;
            }
            k++;
        }
    }
    else {
        goto exit;
    }
    return 0;

exit:
    //not with this firmware
#ifdef exTAPdebug
    TAP_Print("\n%s(), Unsupported firmware: sysid:%d, version:%04x", __func__, fw_sysid, _appl_version);
#endif
    return EXTAPERR_BAD_FIRMWARE;
}

//only for 12.88
int exTAP_GetCurrentGenreStr(int chType, int chNum, char *str)
{
    int ret;
    int nibble1, nibble2;

    ret = exTAP_GetCurrentGenre(chType, chNum, &nibble1, &nibble2);
    if (ret == 0) str = get_content_type_str(nibble1, nibble2);
    return ret;
}

//only for 12.88
int exTAP_GetGenre(TYPE_TapEvent *tapEvtInfo, int *nibble1, int *nibble2)
{
    int i;
    type_eventtable *fw_et;

    *nibble1 = 0;
    *nibble2 = 0;
    if (!fw_max_events) goto exit;
    if (!fw_event_table_size) goto exit;
    if (!fw_event_table) goto exit;
    if ((fw_sysid==456)&&(_appl_version==0x1288)) {
        for (i=0; i<fw_max_events; i++) {
            //the sizeof(fw_event_table) is different from 456:1288
            //so, don't use a static index, e.g. fw_event_table[i].
            fw_et = (type_eventtable *)((dword)fw_event_table + (i * fw_event_table_size));

            //match satIdx, orgNetId, tsId, svcId, evtId,
            if (fw_et->char00 & 0xc0) {
                if (tapEvtInfo->evtId == fw_et->event_id) {
                    if (tapEvtInfo->svcId == fw_et->service_id) {
                        if (tapEvtInfo->tsId == fw_et->transport_stream_id) {
                            if (tapEvtInfo->orgNetId == fw_et->original_network_id) {
                                if (tapEvtInfo->satIdx == fw_et->sat_index) {
                                    //get the genre type
                                    *nibble1 = fw_et->content_nibble1;
                                    *nibble2 = fw_et->content_nibble2;
                                    break;
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return 0;

exit:
    //not with this firmware
#ifdef exTAPdebug
    TAP_Print("\n%s(), Unsupported firmware: sysid:%d, version:%04x", __func__, fw_sysid, _appl_version);
#endif
    return EXTAPERR_BAD_FIRMWARE;
}
//----------------------------------------------------------------------------
